@@ -5,7 +5,6 @@ our (%gui, %vmc, %signal, %prefs);
 
 # Block for filling in guest details
 {
-
     # This two subs might be able to be replaced with a single one
     sub expand_details_row {
         my ($treeview, $treeiter, $treepath) = @_;
@@ -21,19 +20,35 @@ our (%gui, %vmc, %signal, %prefs);
         $prefs{$row[5]} = 0;
     }
 
+    # This two subs might be able to be replaced with a single one
+    # sub expand_guest_row {
+    #    my ($treeview, $treeiter, $treepath) = @_;
+    #    my $model = $treeview->get_model();
+    #    my @row = $model->get($treeiter);
+    #    $prefs{'GROUPEXPAND_' . $row[0]} = 1;
+    # }
+
+    #sub collapse_guest_row {
+    #    my ($treeview, $treeiter, $treepath) = @_;
+    #    my $model = $treeview->get_model();
+    #    my @row = $model->get($treeiter);
+    #    $prefs{'GROUPEXPAND_' . $row[0]} = 0;
+    #}
+
     # Fill a brief version of the guest details
     sub fill_list_details_brief {
         &addrow_log("Fetching guest details...");
         $gui{treestoreDetails}->clear();
         my $treemodel = $gui{treeviewDetails}->get_model();
         my $gref = &getsel_list_guest();
+        my $IGraphicsAdapter = IMachine_getGraphicsAdapter($$gref{IMachine});
         my $iter = &addrow_details(undef, 0, $gui{img}{CatGen}, 1, 'Guest Summary', 3, 800, 4, 0.0, 5, 'EXPANDDETGEN');
         &addrow_details($iter, 1, ' Name:', 2, $$gref{Name});
         &addrow_details($iter, 1, ' Operating System:', 2, IMachine_getOSTypeId($$gref{IMachine}));
         my $mem = IMachine_getMemorySize($$gref{IMachine});
         $mem = ($mem > 1023) ? sprintf("%.2f GB", $mem / 1024) : "$mem MB";
         &addrow_details($iter, 1, ' Base Memory:', 2, $mem);
-        &addrow_details($iter, 1, ' Video Memory:', 2, IMachine_getVRAMSize($$gref{IMachine}) . ' MB');
+        &addrow_details($iter, 1, ' Video Memory:', 2, IGraphicsAdapter_getVRAMSize($IGraphicsAdapter) . ' MB');
         $gui{treeviewDetails}->expand_row($treemodel->get_path($iter), 1) if ($prefs{EXPANDDETGEN});
         my $desciter = &addrow_details(undef, 0, $gui{img}{CatDesc}, 1, 'Description', 3, 800, 4, 0.0, 5, 'EXPANDDETDESC');
         my $desc = IMachine_getDescription($$gref{IMachine});
@@ -54,6 +69,7 @@ our (%gui, %vmc, %signal, %prefs);
         my @IStorageController = IMachine_getStorageControllers($$gref{IMachine});
         my $IAudioAdapter = IMachine_getAudioAdapter($$gref{IMachine});
         my @IUSBController = IMachine_getUSBControllers($$gref{IMachine});
+        my $IGraphicsAdapter = IMachine_getGraphicsAdapter($$gref{IMachine});
         my $geniter = &addrow_details(undef, 0, $gui{img}{CatGen}, 1, 'General', 3, 800, 4, 0.0, 5, 'EXPANDDETGEN');
         &addrow_details($geniter, 1, ' Name:', 2, $$gref{Name});
         &addrow_details($geniter, 1, ' Operating System:', 2, IMachine_getOSTypeId($$gref{IMachine}));
@@ -69,7 +85,7 @@ our (%gui, %vmc, %signal, %prefs);
 
         foreach (1..4) {
             my $bdev = IMachine_getBootOrder($$gref{IMachine}, $_);
-        $bootorder .= "$bdev  " if ($bdev ne 'Null');
+            $bootorder .= "$bdev  " if ($bdev ne 'Null');
         }
 
         $bootorder ? &addrow_details($sysiter, 1, ' Boot Order:', 2, $bootorder) : &addrow_details($sysiter, 1, ' Boot Order:', 2, '<None Enabled>');
@@ -78,19 +94,18 @@ our (%gui, %vmc, %signal, %prefs);
         $vtx .= 'VPID  ' if (IMachine_getHWVirtExProperty($$gref{IMachine}, 'VPID') eq 'true');
         $vtx .= 'PAE/NX  ' if (IMachine_getCPUProperty($$gref{IMachine}, 'PAE') eq 'true');
         $vtx .= 'Nested Paging  ' if (IMachine_getHWVirtExProperty($$gref{IMachine}, 'NestedPaging') eq 'true');
+        $vtx .= 'Nested VT-x/AMD-V  ' if (IMachine_getCPUProperty($$gref{IMachine}, 'HWVirt') eq 'true');
         $vtx ? &addrow_details($sysiter, 1, ' Acceleration:', 2, $vtx) : &addrow_details($sysiter, 1, ' Acceleration:', 2, '<None Enabled>');
         my $paravirt = 'Configured: ' . IMachine_getParavirtProvider($$gref{IMachine}) . ', Effective: ' . IMachine_getEffectiveParavirtProvider($$gref{IMachine});
-        #&addrow_details($sysiter, 1, ' Paravirtualization:', 2, IMachine_getParavirtProvider($$gref{IMachine}));
-
         &addrow_details($sysiter, 1, ' Paravirtualization:', 2, $paravirt);
         $gui{treeviewDetails}->expand_row($treemodel->get_path($sysiter), 1) if ($prefs{EXPANDDETSYS});
 
         my $dispiter = &addrow_details(undef, 0, $gui{img}{CatDisp}, 1, 'Display', 3, 800, 4, 0.0, 5, 'EXPANDDETDISP');
-        &addrow_details($dispiter, 1, ' Video Memory:', 2, IMachine_getVRAMSize($$gref{IMachine}) . ' MB');
-        &addrow_details($dispiter, 1, ' Screens: ', 2, IMachine_getMonitorCount($$gref{IMachine}));
+        &addrow_details($dispiter, 1, ' Video Memory:', 2, IGraphicsAdapter_getVRAMSize($IGraphicsAdapter) . ' MB');
+        &addrow_details($dispiter, 1, ' Screens: ', 2, IGraphicsAdapter_getMonitorCount($IGraphicsAdapter));
         my $vidaccel = '';
-        $vidaccel .= '2D Video  ' if (IMachine_getAccelerate2DVideoEnabled($$gref{IMachine}) eq 'true');
-        $vidaccel .= '3D  ' if (IMachine_getAccelerate3DEnabled($$gref{IMachine}) eq 'true');
+        $vidaccel .= '2D Video  ' if (IGraphicsAdapter_getAccelerate2DVideoEnabled($IGraphicsAdapter) eq 'true');
+        $vidaccel .= '3D  ' if (IGraphicsAdapter_getAccelerate3DEnabled($IGraphicsAdapter) eq 'true');
         $vidaccel ? &addrow_details($dispiter, 1, ' Acceleration:', 2, $vidaccel) : &addrow_details($dispiter, 1, ' Acceleration:', 2, '<None Enabled>');
         IVRDEServer_getEnabled($IVRDEServer) eq 'true' ? &addrow_details($dispiter, 1, ' Remote Display Ports:', 2, IVRDEServer_getVRDEProperty($IVRDEServer, 'TCP/Ports'))
                                                     : &addrow_details($dispiter, 1, ' Remote Display Ports:', 2, '<Remote Display Disabled>');
@@ -302,7 +317,7 @@ sub fill_list_log {
         }
         else {
             my $iter = $gui{'liststoreLog' . $lognum}->append;
-            $gui{'liststoreLog' . $lognum}->set($iter, 0, '', 1, 'This log file does not yet exist.');
+            $gui{'liststoreLog' . $lognum}->set($iter, 0, '', 1, 'This log file does not exist yet');
         }
     }
 }
@@ -358,7 +373,7 @@ sub fill_list_editshared {
 sub fill_list_editstorage {
     my ($IMachine) = @_;
     &busy_pointer($gui{dialogEdit}, 1);
-    &sens_editstor_nosel();
+    &storage_sens_nosel();
     $gui{treestoreEditStor}->clear();
     my @IStorageController = IMachine_getStorageControllers($IMachine);
 
@@ -412,7 +427,7 @@ sub fill_list_editstorage {
             else {
                 $gui{treestoreEditStor}->set($citer, 0,  '<Empty Drive>',               # Display Name
                                                      1,  $$attach{type},                # Display Typee
-                                                     2,  "Empty Drive",  # Tooltip
+                                                     2,  'Empty Drive',  # Tooltip
                                                      3,  0,                             # Is it a controller
                                                      4,  $ctr_attr{bus},                # The bus the medium is on
                                                      5,  $ctr_attr{name},               # The name of the controller it is on
@@ -521,17 +536,20 @@ sub addrow_log {
 
 # Adds a row to the editshared list
 sub addrow_editshared {
-    my ($share, $permanent) = @_;
+    my ($ISharedFolder, $permanent) = @_;
+    my $shrname = ISharedFolder_getName($ISharedFolder);
+    my $shrpath = ISharedFolder_getHostPath($ISharedFolder);
+    my $shrerror = ISharedFolder_getLastAccessError($ISharedFolder);
+    my $shraccessible = ISharedFolder_getAccessible($ISharedFolder);
     my $access = 'Full';
     my $automount = 'No';
-    my $tooltip = "$$share{name}: $$share{hostPath}";
-    $access = 'Read-Only' if ($$share{writable} eq 'false');
-    $automount = 'Yes' if ($$share{autoMount} eq 'true');
-    $tooltip = $$share{lastAccessError} if ($$share{lastAccessError});
-    $tooltip = 'Share is not accessible' if ($$share{accessible} eq 'false');
+    my $tooltip = ($shrerror) ? $shrerror : "$shrname ($shrpath)";
+    $tooltip .= ($shraccessible eq 'false') ? ' : Share is not accessible' : '';
+    $access = 'Read-Only' if (ISharedFolder_getWritable($ISharedFolder) eq 'false');
+    $automount = 'Yes' if (ISharedFolder_getAutoMount($ISharedFolder) eq 'true');
     my $iter = $gui{liststoreEditShared}->append;
-    if ($$share{accessible} eq 'false') { $gui{liststoreEditShared}->set($iter, 0, $$share{name}, 1, $$share{hostPath}, 2, $access, 3, $automount, 4, $gui{img}{Error}, 5, $tooltip, 6, $permanent); }
-    else { $gui{liststoreEditShared}->set($iter, 0, $$share{name}, 1, $$share{hostPath}, 2, $access, 3, $automount, 5, $tooltip, 6, $permanent); }
+    if ($shraccessible eq 'false') { $gui{liststoreEditShared}->set($iter, 0, $shrname, 1, $shrpath, 2, $access, 3, $automount, 4, $gui{img}{Error}, 5, $tooltip, 6, $permanent); }
+    else { $gui{liststoreEditShared}->set($iter, 0, $shrname, 1, $shrpath, 2, $access, 3, $automount, 5, $tooltip, 6, $permanent); }
 }
 
 sub addrow_info {
@@ -546,6 +564,8 @@ sub addrow_ec {
     return $iter;
 }
 
+# Returns the contents of the chosen column of the selected combobox row or
+# returns the row iterator if no column is chosen
 sub getsel_combo {
     my ($widget, $col) = @_;
     my $model = $widget->get_model();
@@ -555,6 +575,23 @@ sub getsel_combo {
     else { return $model->get($iter); }
 }
 
+# Sets the combobox active to the chosen text in the chosen column
+sub combobox_set_active_text {
+    my ($combobox, $txt, $col) = @_;
+    my $i = 0;
+    $combobox->get_model->foreach (
+                            sub {
+                                my ($model, $path, $iter) = @_;
+                                if ($txt eq $model->get_value($iter, $col)) {
+                                    ($i) = $path->get_indices;
+                                    return 1; # stop
+                                }
+                                return 0; # continue
+                            }
+                          );
+    $combobox->set_active($i);
+}
+
 # Handles single and multiple selections
 sub getsel_list_remotefiles {
     my @filearray;
@@ -709,6 +746,7 @@ sub onsel_list_editstorage {
         if ($$storref{Bus} eq 'IDE') { $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorIDECtrType}); }
         elsif ($$storref{Bus} eq 'USB') { $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorUSBCtrType}); }
         elsif ($$storref{Bus} eq 'SCSI') { $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorSCSICtrType}); }
+        elsif ($$storref{Bus} eq 'VirtioSCSI') { $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorVirtioSCSICtrType}); }
         elsif ($$storref{Bus} eq 'SATA') {
             $gui{labelEditStorPortCount}->show();
             $gui{spinbuttonEditStorPortCount}->show();
@@ -741,7 +779,7 @@ sub onsel_list_editstorage {
             $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorFloppyCtrType});
         }
 
-        &combobox_set_active_text($gui{comboboxEditStorCtrType}, $variant);
+        &combobox_set_active_text($gui{comboboxEditStorCtrType}, $variant, 0);
         $gui{comboboxEditStorCtrType}->signal_handler_unblock($signal{stortype});
     }
     else { # This is a medium, not a controller
@@ -832,6 +870,10 @@ sub onsel_list_editstorage {
             $gui{comboboxEditStorDevPort}->set_model($gui{liststoreEditStorDevPortUSB});
             $gui{comboboxEditStorDevPort}->set_active($$storref{Port});
         }
+        elsif ($$storref{Bus} eq 'VirtioSCSI') {
+            $gui{comboboxEditStorDevPort}->set_model($gui{liststoreEditStorDevPortVirtio});
+            $gui{comboboxEditStorDevPort}->set_active($$storref{Port});
+        }
     }
 }
 
@@ -891,8 +933,10 @@ sub onsel_list_editstorage {
         my $iter = $gui{treeviewVMMFloppy}->get_selection->get_selected() ? $gui{treeviewVMMFloppy}->get_selection->get_selected() : $model->get_iter_first();
         my @row = $model->get($iter);
         $selected{$_} = shift @row foreach ('Name', 'IMedium', 'Size', 'Accessible', 'Tooltip', 'Location', 'Type');
-        $gui{toolbuttonVMMCopy}->set_sensitive(0);
+        $gui{toolbuttonVMMCopy}->set_sensitive(1);
+        $gui{toolbuttonVMMMove}->set_sensitive(1);
         $gui{toolbuttonVMMModify}->set_sensitive(0);
+        $gui{toolbuttonVMMCompact}->set_sensitive(0);
 
         my $gnames;
         my @mids = IMedium_getMachineIds($selected{IMedium});
@@ -990,7 +1034,9 @@ sub onsel_list_editstorage {
         my @row = $model->get($iter);
         $selected{$_} = shift @row foreach ('Name', 'IMedium', 'Size', 'Accessible', 'Tooltip', 'Location', 'Type');
         $gui{toolbuttonVMMCopy}->set_sensitive(0);
+        $gui{toolbuttonVMMMove}->set_sensitive(1);
         $gui{toolbuttonVMMModify}->set_sensitive(0);
+        $gui{toolbuttonVMMCompact}->set_sensitive(0);
 
         my $gnames;
         my @mids = IMedium_getMachineIds($selected{IMedium});
@@ -1140,9 +1186,11 @@ sub onsel_list_editstorage {
         my $model = $gui{treeviewVMMHD}->get_model();
         my $iter = $gui{treeviewVMMHD}->get_selection->get_selected() ? $gui{treeviewVMMHD}->get_selection->get_selected() : $model->get_iter_first();
         my @row = $model->get($iter);
-        $selected{$_} = shift @row foreach ('Name', 'IMedium', 'Asize', 'Vsize', 'Accessible', 'Tooltip', 'Location', 'Type');
+        $selected{$_} = shift @row foreach ('Name', 'IMedium', 'Asize', 'Vsize', 'Accessible', 'Tooltip', 'Location', 'Type', 'LsizeInt');
         $gui{toolbuttonVMMCopy}->set_sensitive(1);
+        $gui{toolbuttonVMMMove}->set_sensitive(1);
         $gui{toolbuttonVMMModify}->set_sensitive(1);
+        $gui{toolbuttonVMMCompact}->set_sensitive(1);
 
         my $gnames;
         my @mids = IMedium_getMachineIds($selected{IMedium});
@@ -1211,7 +1259,8 @@ sub onsel_list_editstorage {
                                     4, $gui{img}{Error},
                                     5, $mattr{accesserr},
                                     6, $mattr{location},
-                                    7, $mattr{type});
+                                    7, $mattr{type},
+                                    8, $mattr{logsize});
         }
         else {
             $treestore->set($citer, 0, $mattr{name},
@@ -1220,7 +1269,8 @@ sub onsel_list_editstorage {
                                     3, &bytesToX($mattr{logsize}),
                                     5, $mattr{location}, # Tooltip can be location
                                     6, $mattr{location},
-                                    7, $mattr{type});
+                                    7, $mattr{type},
+                                    8, $mattr{logsize});
         }
 
         if (($IMedium eq $selected{IMedium})) {
@@ -1240,13 +1290,12 @@ sub set_vmm_fields {
     $gui{labelVMMAttachedToField}->set_text($gnames);
     $gui{labelVMMLocationField}->set_text($$selected{Location});
 
-    if (&imedium_has_property($$selected{IMedium}, 'CRYPT/KeyId')) { $gui{labelVMMEncryptedField}->set_text(IMedium_getProperty($$selected{IMedium}, 'CRYPT/#KeyId')); }
-    else { $gui{labelVMMEncryptedField}->set_text('Not Encrypted'); }
+    if (&imedium_has_property($$selected{IMedium}, 'CRYPT/KeyId')) { $gui{labelVMMEncryptedField}->set_text(IMedium_getProperty($$selected{IMedium}, 'CRYPT/KeyId')); }
+    else { $gui{labelVMMEncryptedField}->set_text('<Not Encrypted>'); }
 
     $gui{labelVMMUUIDField}->set_text(IMedium_getId($$selected{IMedium}));
 }
 
-
 sub clr_list_vmm {
     my ($treestore) = @_;
     &vmm_sens_unselected(); # Do whenever list is cleared
@@ -1298,7 +1347,7 @@ sub onsel_list_shared {
             $gui{treeviewSnapshots}->expand_all();
         }
 
-        &addrow_log('Snapshot fetch complete.');
+        &addrow_log('Snapshot fetch complete');
     }
 
     # Clear snapshot list and set sensitivity
@@ -1522,10 +1571,9 @@ sub onsel_list_shared {
         }
 
         &busy_pointer($gui{windowMain}, 0);
-        &addrow_log('Guest list complete.');
+        &addrow_log('Guest list complete');
         &addrow_log('WARNING: You have one or more guests that are inaccessible and have been excluded from ' .
                     'the guest list. You must use vboxmanage or VirtualBox on the server to fix these issues') if ($inaccessible);
-
         &update_server_membar();
     }
 
@@ -1553,7 +1601,7 @@ sub onsel_list_shared {
                                                 3, $password);
         }
         else {
-            $pname = 'Unnamed' . int(rand(9999));
+            $pname = 'Unnamed-' . int(rand(999999));
             $gui{liststoreProfiles}->set($iter, 0, $pname,
                                             1, 'http://localhost:18083',
                                             2, '',
@@ -1635,9 +1683,6 @@ sub onsel_list_shared {
             $prefs{AUTOCONNPROF} = '' if ( $prefs{AUTOCONNPROF} eq $gui{entryPrefsConnectionProfileName}->get_text() );
         }
     }
-
 }
 
-
-
 1;
