@@ -3,7 +3,7 @@ use strict;
 use warnings;
 our (%gui, %signal, %prefs);
 
-sub show_dialog_newguest {
+sub show_dialog_new_guest {
     my $vhost = &vhost();
     my ($osfam, $osver) = &osfamver();
     $gui{checkbuttonNewNewHD}->set_active(1);
@@ -11,9 +11,9 @@ sub show_dialog_newguest {
     $gui{radiobuttonNewDynamic}->set_sensitive(1);
     $gui{radiobuttonNewFixed}->set_sensitive(1);
     $gui{radiobuttonNewSplit}->set_sensitive(0);
-    $gui{entryNewName}->set_text('NewGuest' . int(rand(9999)));
-    $gui{spinbuttonNewMemory}->set_range(4, $$vhost{memsize});
-    $gui{spinbuttonNewNewHDSize}->set_range(8.00, int($$vhost{maxhdsize} / 1048576));
+    $gui{entryNewName}->set_text('NewGuest-' . int(rand(999999)));
+    $gui{spinbuttonNewMemory}->set_range($$vhost{minguestram}, $$vhost{memsize});
+    $gui{spinbuttonNewNewHDSize}->set_range($$vhost{minhdsizemb}, $$vhost{maxhdsizemb});
     $gui{comboboxNewOSFam}->signal_handler_block($signal{fam}); # Block to avoid signal emission when changing
     $gui{comboboxNewOSVer}->signal_handler_block($signal{ver});
     $gui{liststoreNewOSFam}->clear();
@@ -58,45 +58,61 @@ sub show_dialog_newguest {
             else {
                 $gui{dialogNew}->hide;
 
-                my %new = (name => $gui{entryNewName}->get_text(),
-                           fam  => &getsel_combo($gui{comboboxNewOSFam}, 1),
-                           ver  => &getsel_combo($gui{comboboxNewOSVer}, 1),
-                           mem  => $gui{spinbuttonNewMemory}->get_value_as_int());
+                my $name = $gui{entryNewName}->get_text();
+                my $IMachine = IVirtualBox_createMachine($gui{websn}, '', $name, '', &getsel_combo($gui{comboboxNewOSVer}, 1), 'UUID 00000000-0000-0000-0000-000000000000', '');
 
-                my ($IMachine, $dvdctrname, $hdctrname, $floppyctrname) = &create_new_guest(\%new);
+                if ($IMachine) {
+                    IMachine_applyDefaults($IMachine);
+                    IMachine_setMemorySize($IMachine, $gui{spinbuttonNewMemory}->get_value_as_int());
 
-                if ($gui{checkbuttonNewNewHD}->get_active() == 1) {
+                    my $IAudioAdapter = IMachine_getAudioAdapter($IMachine);
+                    IAudioAdapter_setAudioDriver($IAudioAdapter, 'Null');
+                    my $IVRDEServer = IMachine_getVRDEServer($IMachine);
+                    IVRDEServer_setVRDEProperty($IVRDEServer, 'VideoChannel/Quality', 75);
+                    IVRDEServer_setEnabled($IVRDEServer, 'true');
+                    IVRDEServer_setAllowMultiConnection($IVRDEServer, 'true');
+                    IMachine_setClipboardMode($IMachine, 'Bidirectional');
 
-                    if ($IMachine) {
+                    if ($$vhost{vrdeextpack} =~ m/vnc/i) { IVRDEServer_setVRDEProperty($IVRDEServer, 'TCP/Ports', $prefs{DEFVNCPORTS}) }
+                    else { IVRDEServer_setVRDEProperty($IVRDEServer, 'TCP/Ports', $prefs{DEFRDPPORTS}); }
+
+                    IMachine_saveSettings($IMachine);
+                    IVirtualBox_registerMachine($gui{websn}, $IMachine);
+                    &addrow_log("Created a new guest: $name");
+
+                    if ($gui{checkbuttonNewNewHD}->get_active() == 1) {
                         my $sref = &get_session($IMachine);
 
                         if ($$sref{Type} eq 'WriteLock') {
                             my $IMediumHD;
 
                             if ($gui{radiobuttonNewNewHD}->get_active()) {
-                                my %newhd = (diskname   => $new{name}, # Use guest name as basis for disk
+                                my %newhd = (diskname   => $name, # Use guest name as basis for disk
+                                             devicetype => 'HardDisk',
+                                             mode       => 'ReadWrite',
                                              size       => $gui{spinbuttonNewNewHDSize}->get_value_as_int() * 1048576,
-                                             allocation => 'Standard', # Standard == Dynamic Allocation
+                                             allocation => ['Standard'], # Standard == Dynamic Allocation
                                              imgformat  => &getsel_combo($gui{comboboxNewNewFormat}, 1),
-                                             location   => &rcatdir($$vhost{machinedir}, $new{name}));
+                                             location   => &rcatdir($$vhost{machinedir}, $name));
 
-                                if ($gui{radiobuttonNewFixed}->get_active()) { $newhd{allocation} = 'Fixed'; }
-                                elsif ($gui{radiobuttonNewSplit}->get_active()) { $newhd{allocation} = 'VmdkSplit2G'; }
-                                $IMediumHD = &create_new_hd(\%newhd, $gui{windowMain});
+                                if ($gui{radiobuttonNewFixed}->get_active()) { $newhd{allocation} = ['Fixed']; }
+                                elsif ($gui{radiobuttonNewSplit}->get_active()) { $newhd{allocation} = ['VmdkSplit2G']; }
+                                $IMediumHD = &create_new_dskimg(\%newhd);
                             }
                             else { $IMediumHD = &getsel_combo($gui{comboboxNewChooseHD}, 1); }
 
-                            my $IStorCtrHD = IMachine_getStorageControllerByName($$sref{IMachine}, $hdctrname);
+                            my %os = %{ $$osver{&getsel_combo($gui{comboboxNewOSVer}, 1)} };
+                            my $IStorCtrHD = IMachine_getStorageControllerByInstance($$sref{IMachine}, $os{recommendedHDStorageBus}, 0);
                             my %hdaddress = &get_free_deviceport($$sref{IMachine}, $IStorCtrHD);
-                            IMachine_attachDevice($$sref{IMachine}, $hdctrname, $hdaddress{portnum}, $hdaddress{devnum}, 'HardDisk', $IMediumHD) if ($IMediumHD);
-                            my $IStorCtrDVD = IMachine_getStorageControllerByName($$sref{IMachine}, $dvdctrname); # Attach Empty CD/DVD Device
+                            IMachine_attachDevice($$sref{IMachine}, $os{recommendedHDStorageBus}, $hdaddress{portnum}, $hdaddress{devnum}, 'HardDisk', $IMediumHD) if ($IMediumHD);
+                            my $IStorCtrDVD = IMachine_getStorageControllerByInstance($$sref{IMachine}, $os{recommendedDVDStorageBus}, 0); # Attach Empty CD/DVD Device
                             my %dvdaddress = &get_free_deviceport($$sref{IMachine}, $IStorCtrDVD);
-                            IMachine_attachDevice($$sref{IMachine}, $dvdctrname, $dvdaddress{portnum}, $dvdaddress{devnum}, 'DVD', '');
+                            IMachine_attachDevice($$sref{IMachine}, $os{recommendedDVDStorageBus}, $dvdaddress{portnum}, $dvdaddress{devnum}, 'DVD', '');
 
-                            if ($floppyctrname) {
-                                my $IStorCtrFloppy = IMachine_getStorageControllerByName($$sref{IMachine}, $floppyctrname);
+                            if ($os{recommendedFloppy} eq 'true') {
+                                my $IStorCtrFloppy = IMachine_getStorageControllerByInstance($$sref{IMachine}, 'Floppy', 0);
                                 my %floppyaddress = &get_free_deviceport($$sref{IMachine}, $IStorCtrFloppy);
-                                IMachine_attachDevice($$sref{IMachine}, $floppyctrname, $floppyaddress{portnum}, $floppyaddress{devnum}, 'Floppy', '');
+                                IMachine_attachDevice($$sref{IMachine}, 'Floppy', $floppyaddress{portnum}, $floppyaddress{devnum}, 'Floppy', '');
                             }
 
                             IMachine_saveSettings($$sref{IMachine});
@@ -104,10 +120,9 @@ sub show_dialog_newguest {
 
                         ISession_unlockMachine($$sref{ISession}) if (ISession_getState($$sref{ISession}) eq 'Locked');
                     }
-                    else { &show_err_msg('createguest', " ($new{name})"); }
+                    &fill_list_guest();
                 }
-
-                &fill_list_guest();
+                else { &show_err_msg('createguest', " ($name)"); }
             }
         }
         else { $gui{dialogNew}->hide; }
@@ -115,16 +130,10 @@ sub show_dialog_newguest {
     } until (!$gui{dialogNew}->visible());
 }
 
-# We have some serious limitations as the moment when setting the clone
-# options. The vboxservice.pm doesn't seem to be able to handle the
-# passing of arrays which means when creating a clone we can only set
-# one option at creation. Ie we can't have Linked and KeepALLMACs set.
-# Need to find a work around.
+# Shows the dialog for creating a new clone
 sub show_dialog_clone {
     my $gref = &getsel_list_guest();
-    $gui{entryCloneName}->set_text($$gref{Name} . ' Clone' . int(rand(9999)));
-    $gui{comboboxCloneType}->set_active(0);
-    $gui{checkbuttonCloneNewMACs}->set_active(1);
+    $gui{entryCloneName}->set_text($$gref{Name} . '-Clone-' . int(rand(999999)));
 
     do {
         my $response = $gui{dialogClone}->run;
@@ -134,13 +143,25 @@ sub show_dialog_clone {
             if (!$gui{entryCloneName}->get_text()) { &show_err_msg('invalidname'); }
             else {
                 $gui{dialogClone}->hide;
-                my $clonetype = 'Full'; # Defaults to Full
-                my $newmacs = $gui{checkbuttonCloneNewMACs}->get_active();
 
-                if ($gui{comboboxCloneType}->get_active() == 2) { $clonetype = 'Link'; }
-                elsif ($gui{comboboxCloneType}->get_active() == 1) { $clonetype = 'Current'; }
+                my %newclone = (name    => $gui{entryCloneName}->get_text(),
+                                mode    => 'MachineState',
+                                linked  => 0,
+                                options => []);
 
-                &create_new_clone($gref, $gui{entryCloneName}->get_text(), $clonetype, $newmacs);
+                # All other clone types just use 'MachineState'
+                $newclone{mode} = 'AllStates' if ($gui{comboboxCloneType}->get_active() == 0);
+
+                # The Link type is not really a mode, but an option
+                if ($gui{comboboxCloneType}->get_active() == 2) {
+                    push(@{$newclone{options}}, 'Link');
+                    $newclone{linked} = 1;
+                }
+                push(@{$newclone{options}}, 'KeepAllMACs') if ($gui{comboboxCloneMAC}->get_active() == 0); # New MACs are generated by default
+                push(@{$newclone{options}}, 'KeepNATMACs') if ($gui{comboboxCloneMAC}->get_active() == 1); # New MACs are generated by default
+                push(@{$newclone{options}}, 'KeepDiskNames') if ($gui{checkbuttonCloneKeepDiskNames}->get_active() == 1);
+                push(@{$newclone{options}}, 'KeepHwUUIDs') if ($gui{checkbuttonCloneKeepHardwareUUIDs}->get_active() == 1);
+                &create_new_clone($gref, \%newclone);
                 &fill_list_guest();
             }
         }
@@ -195,127 +216,86 @@ sub get_free_deviceport {
     return %address;
 }
 
+# Creates a clone of a guest
 sub create_new_clone {
     my $vhost = &vhost();
-    my ($srcgref, $clonename, $clonetype, $newmacs) = @_;
-
+    my ($srcgref, $cloneref) = @_;
     # We create a new 'empty' guest
-    my $cloneIMachine = IVirtualBox_createMachine($gui{websn}, '', $clonename, '', $$srcgref{Osid}, 'UUID 00000000-0000-0000-0000-000000000000', '');
+    my $cloneIMachine = IVirtualBox_createMachine($gui{websn}, '', $$cloneref{name}, '', $$srcgref{Osid}, 'UUID 00000000-0000-0000-0000-000000000000', '');
     my $IProgress;
 
-    if ($clonetype eq 'Link') { # Is cancellable
-        # Force newmacs to be 0 because they get regenerated automatically anyway (whether we like it or not)
-        $newmacs = 0;
+    if ($$cloneref{linked}) { # Is cancellable
         # Linked clones require a snapshot of the source to be taken first. The IMachine must be of that snapshot
-        &take_snapshot("Base for $$srcgref{Name} and $clonename", "Snapshot automatically taken when cloning $$srcgref{Name} to $clonename");
+        &take_snapshot("Base for $$srcgref{Name} and $$cloneref{name}", "Snapshot automatically taken when cloning $$srcgref{Name} to $$cloneref{name}");
         my $snapIMachine = ISnapshot_getMachine(IMachine_getCurrentSnapshot($$srcgref{IMachine}));
-        $IProgress = IMachine_cloneTo($snapIMachine, $cloneIMachine, 'MachineState', 'Link');
+        $IProgress = IMachine_cloneTo($snapIMachine, $cloneIMachine, $$cloneref{mode}, @{$$cloneref{options}})
+    }
+    else {
+        $IProgress = IMachine_cloneTo($$srcgref{IMachine}, $cloneIMachine, $$cloneref{mode}, @{$$cloneref{options}});
     }
-    # For some reason if you don't specify an option it always assumes linked so we'll use KeepAllMacs then regenerate them
-    elsif ($clonetype eq 'Current') { $IProgress = IMachine_cloneTo($$srcgref{IMachine}, $cloneIMachine, 'MachineState', 'KeepAllMACs'); }
-    else { $IProgress = IMachine_cloneTo($$srcgref{IMachine}, $cloneIMachine, 'AllStates', 'KeepAllMACs'); }
 
-    &show_progress_window($IProgress, 'Cloning Guest') if ($IProgress); # MUST NOT USE $cloneIMachine until progress is complete, otherwise it waits
+    &show_progress_window($IProgress, 'Cloning Guest', $gui{img}{ProgressClone}) if ($IProgress); # MUST NOT USE $cloneIMachine until progress is complete, otherwise it waits
 
     if (IProgress_getCanceled($IProgress) eq 'true') {
-        &addrow_log('Cancelled guest clone');
+        &addrow_log("Cancelled creation of the clone $$cloneref{name}");
         IManagedObjectRef_release($cloneIMachine);
         $cloneIMachine = undef;
     }
     else {
-        # Regenerate the MACs if the option has been selected
-        if ($newmacs) {
-            foreach (0..($$vhost{maxnet}-1)) {
-                my $INetworkAdapter = IMachine_getNetworkAdapter($cloneIMachine, $_);
-                INetworkAdapter_setMACAddress($INetworkAdapter, IHost_generateMACAddress($$vhost{IHost}));
-            }
-        }
-
         IMachine_saveSettings($cloneIMachine);
         IVirtualBox_registerMachine($gui{websn}, $cloneIMachine);
-        &addrow_log("Cloned $$srcgref{Name} to $clonename");
+        &addrow_log("Cloned $$srcgref{Name} to $$cloneref{name}");
     }
 }
 
-# Creates a new guest
-sub create_new_guest {
-    my ($newref) = @_;
-    my $osver = &osver();
-    my $vhost = &vhost();
-    my %os = %{ $$osver{$$newref{ver}} };
-    my $dvdctrname = $os{recommendedDVDStorageBus};
-    my $hdctrname = $os{recommendedHDStorageBus};
-    my $floppyctrname = 'Floppy' if ($os{recommendedFloppy} eq 'true');
-    my $IMachine = IVirtualBox_createMachine($gui{websn}, '', $$newref{name}, '', $$newref{ver}, 'UUID 00000000-0000-0000-0000-000000000000', '');
-
-    if ($IMachine) {
-        my $IVRDEServer = IMachine_getVRDEServer($IMachine);
-        my $IBIOSSettings = IMachine_getBIOSSettings($IMachine);
-        my $IAudioAdapter = IMachine_getAudioAdapter($IMachine);
-        IMachine_setChipsetType($IMachine, $os{recommendedChipset});
-        IBIOSSettings_setIOAPICEnabled($IBIOSSettings, &bl($os{recommendedIOAPIC}));
-        IMachine_setCPUProperty($IMachine, 'PAE', &bl($os{recommendedPAE}));
-        IMachine_setHWVirtExProperty($IMachine, 'Enabled', 1);
-        IMachine_setHPETEnabled($IMachine, &bl($os{recommendedHPET}));
-        IMachine_setVRAMSize($IMachine, $os{recommendedVRAM});
-        IMachine_setFirmwareType($IMachine, $os{recommendedFirmware});
-        IMachine_setMemorySize($IMachine, $$newref{mem});
-        IMachine_setRTCUseUTC($IMachine, &bl($os{recommendedRTCUseUTC}));
-        IMachine_addUSBController($IMachine, 'OHCI', 'OHCI') if (&bl($os{recommendedUSB}));
-        IMachine_addUSBController($IMachine, 'EHCI', 'EHCI') if (&bl($os{recommendedUSB}));
-        IMachine_setPointingHIDType($IMachine, 'USBMouse') if ($os{recommendedUSBHID} eq 'true');
-        IMachine_setPointingHIDType($IMachine, 'USBTablet') if ($os{recommendedUSBTablet} eq 'true');
-        IMachine_setAccelerate2DVideoEnabled($IMachine, $os{recommended2DVideoAcceleration});
-        IAudioAdapter_setAudioController($IAudioAdapter, $os{recommendedAudioController});
-        IAudioAdapter_setAudioDriver($IAudioAdapter, 'Null');
-        IAudioAdapter_setEnabled($IAudioAdapter, 1);
-        IAudioAdapter_setEnabledOut($IAudioAdapter, 1);
-
-        my $IStorCtrFloppy = IMachine_addStorageController($IMachine, $floppyctrname, 'Floppy') if ($floppyctrname);
-        my $IStorCtrDVD = IMachine_addStorageController($IMachine, $dvdctrname, $os{recommendedDVDStorageBus});
-        IStorageController_setControllerType($IStorCtrDVD, $os{recommendedDVDStorageController});
-        IStorageController_setPortCount($IStorCtrDVD, IStorageController_getMinPortCount($IStorCtrDVD)); # Controllers have all ports on by default. Set to the minimum
-
-        if ($os{recommendedHDStorageBus} ne $os{recommendedDVDStorageBus}) {
-            my $IStorCtrHD = IMachine_addStorageController($IMachine, $hdctrname, $os{recommendedHDStorageBus});
-            IStorageController_setControllerType($IStorCtrHD, $os{recommendedHDStorageController});
-            IStorageController_setPortCount($IStorCtrHD, IStorageController_getMinPortCount($IStorCtrHD)); # Controllers have all ports on by default. Set to the minimum
-        }
-
-        foreach my $slot (0..7) {
-            my $INetworkAdapter = IMachine_getNetworkAdapter($IMachine, $slot);
-            INetworkAdapter_setAdapterType($INetworkAdapter, $os{adapterType});
-        }
-
-        IVRDEServer_setVRDEProperty($IVRDEServer, 'VideoChannel/Quality', 75);
-        if ($$vhost{vrdeextpack} =~ m/vnc/i) { IVRDEServer_setVRDEProperty($IVRDEServer, 'TCP/Ports', $prefs{DEFVNCPORTS}) }
-        else { IVRDEServer_setVRDEProperty($IVRDEServer, 'TCP/Ports', $prefs{DEFRDPPORTS}); }
-        IVRDEServer_setEnabled($IVRDEServer, 'true');
-        IVRDEServer_setAllowMultiConnection($IVRDEServer, 'true');
-        IMachine_saveSettings($IMachine);
-        IVirtualBox_registerMachine($gui{websn}, $IMachine);
-        &addrow_log("Created a new guest: $$newref{name}.");
-    }
-
-    return $IMachine, $dvdctrname, $hdctrname, $floppyctrname;
-}
-
-# Creates a new hard disk and shows a progress window
-sub create_new_hd {
-    my ($newref) = @_;
-    my $ext = 'hdd';
-    # All formats use an extension the same as their name except parallels
-    $ext = $$newref{imgformat} unless ($$newref{imgformat} eq 'parallels');
-    my $IMedium = IVirtualBox_createMedium($gui{websn}, $$newref{imgformat}, &rcatfile($$newref{location}, "$$newref{diskname}.$ext"), 'ReadWrite', 'HardDisk');
-    my $IProgress = IMedium_createBaseStorage($IMedium, $$newref{size}, $$newref{allocation});
-    &show_progress_window($IProgress, 'Creating Hard Disk');
+# Creates a clone of a disk image
+sub create_new_dskimg_clone {
+    my ($newref, $srcIMedium) = @_;
+    $$newref{imgformat} = lc($$newref{imgformat}); # Force it to be lower case
+    my $fileext = &fileext();
+    my $imgfile = &rcatfile($$newref{location}, "$$newref{diskname}.$$fileext{$$newref{imgformat}}");
+    my $IMedium = IVirtualBox_createMedium($gui{websn}, $$newref{imgformat}, $imgfile, $$newref{mode}, $$newref{devicetype});
+    my $IProgress = IMedium_cloneTo($srcIMedium, $IMedium, @{$$newref{allocation}}, undef);
+    &show_progress_window($IProgress, 'Copying or Cloning Disk Image', $gui{img}{ProgressMediaCreate});
 
     if (IProgress_getCanceled($IProgress) eq 'true') {
-        &addrow_log("Cancelled hard disk creation");
+        &addrow_log("Cancelled disk copy or cloning: $imgfile");
         IManagedObjectRef_release($IMedium);
         $IMedium = undef;
     }
-    else { &addrow_log("Created new hard disk: " . &rcatfile($$newref{location}, "$$newref{diskname}.$ext")); }
+    elsif (IMedium_refreshState($IMedium) eq 'NotCreated') {
+        &show_err_msg('diskimgcreation', "Disk Image: $imgfile");
+        IManagedObjectRef_release($IMedium);
+        $IMedium = undef;
+    }
+    else { &addrow_log("Copied or cloned disk image to new file: $imgfile"); }
+
+    Gtk2->main_iteration() while Gtk2->events_pending();
+    return $IMedium;
+}
+
+# Creates a new hard disk or floppy image and shows a progress window
+sub create_new_dskimg {
+    my ($newref) = @_;
+    $$newref{imgformat} = lc($$newref{imgformat}); # Force it to be lower case
+    my $fileext = &fileext();
+    my $imgfile = &rcatfile($$newref{location}, "$$newref{diskname}.$$fileext{$$newref{imgformat}}");
+    my $IMedium = IVirtualBox_createMedium($gui{websn}, $$newref{imgformat}, $imgfile, $$newref{mode}, $$newref{devicetype});
+    # Note: Modified in the vboxService.pm. We expect an array reference in $$newref{allocation}
+    my $IProgress = IMedium_createBaseStorage($IMedium, $$newref{size}, @{$$newref{allocation}});
+    &show_progress_window($IProgress, 'Creating Disk Image', $gui{img}{ProgressMediaCreate});
+
+    if (IProgress_getCanceled($IProgress) eq 'true') {
+        &addrow_log("Cancelled disk image creation: $imgfile");
+        IManagedObjectRef_release($IMedium);
+        $IMedium = undef;
+    }
+    elsif (IMedium_refreshState($IMedium) eq 'NotCreated') {
+        &show_err_msg('diskimgcreation', "Disk Image: $imgfile");
+        IManagedObjectRef_release($IMedium);
+        $IMedium = undef;
+    }
+    else { &addrow_log("Created new disk image: $imgfile"); }
 
     Gtk2->main_iteration() while Gtk2->events_pending();
     return $IMedium;
